/***********************************************************************
* FILENAME :        ctrl_task.c 
*
* DESCRIPTION :
*        FreeRTOS Controller Task 
*
*
* licensed under the Creative Commons Attribution-NonCommercial 4.0 
* 
* AUTHOR :    Romulo Antao        START DATE :    27 MAR 17
*
************************************************************************/

/* Standard includes. */
#include <stdio.h>

/* Kernel includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "queue.h"

/* Hardware includes. */
#include "board.h"
#include <math.h>

/* Misc includes */
#include "t2fls.h"
#include "gpc.h"
#include "matrix.h"

/* Flow control semaphores */
extern SemaphoreHandle_t  CTRL_CTR; // Controller clear to receive
extern SemaphoreHandle_t  CTRL_RTS;	// Controller ready to send

/* Process variables */
float du 	  =  0;	  /* Past increment */
float du_U    =  0;   /* Past increment */
float du_L    =  0;   /* Past increment */
float new_u   =  0;   /* new control signal */
float y_hat   =  0;   /* Estimated model output */

/* Model Related variables */
extern Matrix bU;
extern Matrix bL;
extern Matrix aU_tilde;
extern Matrix aL_tilde;
extern Matrix u;

/* Controller function */
void prvController(void *pvParameters)
{

	// Just to prevent compiler warnings about the unused parameter.
	( void ) pvParameters;

	// Initialize GPC matrices
	gpc_init();

	// Initialize Type-2 Fuzzy Model parameters
    t2fls_init();

	for(;;)
	{
		/* The execution rate is defined by the message exchange
		   between MCU and MATLAB */

		/* Wait until new data from the model is available */
		/* Connect to adc task */
		xSemaphoreTake( CTRL_CTR, portMAX_DELAY );

		/* Do the math! */
		t2fls_PHI();

		/* Iterate the TS Fuzzy Model */
		y_hat = t2fls_iterate();
	
		/* Linearize the model at the current iteration */
		t2fls_linearize();

		/* Get the control signal */
		du_U = gpc(&aU_tilde, &bU, du, lambda);
		du_L = gpc(&aL_tilde, &bL, du, lambda);
		
		du = (du_L + du_U) / 2;
		new_u = u.matrix[0][0] + du;

		/* Check the saturation bounds */
		if (new_u < U_SAT_L)
			new_u = U_SAT_L;

		if (new_u > U_SAT_U)
			new_u = U_SAT_U;

		/* Update the regressor */
		gpc_uShift(new_u);

		/* Signal UDP TX task that the controller processing is finished */
		xSemaphoreGive( CTRL_RTS );
		/* connect with the actuation task */

	}

}

