/***********************************************************************
* FILENAME :        gpc.h 
*
* DESCRIPTION :
*        GPC algorithm implementation
*
*
* licensed under the Creative Commons Attribution-NonCommercial 4.0 
* 
* AUTHOR :    Romulo Antao        START DATE :    27 MAR 17
*
************************************************************************/

#ifndef GPC_H
#define GPC_H

/* Standard includes. */
#include <stdio.h>

/* Misc Includes */
#include "matrix.h"
#include "t2fls.h"

/* GPC parametrization */
#define Np   10 		/* Prediction Horizon*/
#define Nu   3  		/* Control Horizon */
#define ng   Np+nb-1 	/* Dimension of g matrix: Np+nb-1 */
#define lambda 0.01 	/* Control Activity Penalty */

#define U_SAT_L 0		/* Lower bound of the saturation signal */
#define U_SAT_U 200 	/* Upper bound of the saturation signal */

/* Implementation of the linear GPC */
int   gpc_init(void);

float gpc(Matrix *a_tilde, Matrix *b, float du, float penalty);
int   gpc_K(void);
float gpc_du(void);
int   gpc_yShift(float new_data);
int   gpc_uShift(float new_data);
int   gpc_rShift(float new_data);
int   gpc_regInit(float r_init[], float y_init[], float u_init[]);

#endif