/***********************************************************************
* FILENAME :        t2fls.h 
*
* DESCRIPTION :
*        GPC algorithm implementation with Type 2 Fuzzy Logic
*
*
* licensed under the Creative Commons Attribution-NonCommercial 4.0 
* 
* AUTHOR :    Romulo Antao        START DATE :    27 MAR 17
*
************************************************************************/

#ifndef MODEL_H
#define MODEL_H

/* Standard includes. */
#include <stdio.h>

/* Misc Includes */
#include "matrix.h"

/* Consequent part model structure */
/* a = [1 -a1 -a2] *
//* b = [b1 b2] */
/* a_tilde = [a 0] - [0 a] */
/* a_tilde = [1 -a1+1 -a2+a1 a2] */ 

/* Model parametrization */
#define na  2 /* Number of denominator parameters */
#define nb  2 /* Number of numerator parameters */

#define R   5 /* Number of Rules */
#define NA  2 /* Number of Antecedents */ 
#define NC  na+nb  /* Number of parameters at the consequent linear function */

/* Implementation of Type-2 Fuzzy Model inference */
int t2fls_init(void);

float t2fls_iterate(void);
int   t2fls_linearize(void);

float typereduction_KM(void);
void  gauss_t2mf(float input, float M1, float M2, float sigma, float *uu, float *ll);
float gaussmf(float x, float c, float sigma);
void  sort(Matrix *Msrc, int s_idx[]);

int   t2fls_M1(float new_data[][NA]);
int   t2fls_M2(float new_data[][NA]);
int   t2fls_sigma(float new_data[][NA]);
int   t2fls_C(float new_data[][NC]);
int   t2fls_S(float new_data[][NC]);

int   t2fls_PHI(void);

float t1fls_iterate(void);
int t1fls_linearize(void);



#endif