function [Theta, PM, PredError] = rlsdf(Theta, Phi, PM, y, npar, ForgFactor)

%   RLSDF.M
%
%   Alexandre Manuel Mota - Outubro de 1997
%
%   Last Revision - 04/October/2007
%
%   [Theta, PM, PredError] = rlsdf(Theta, Phi, PM, y, npar, ForgFactor)
%
%   Theta - estimated parameter
%   PM - Covariance matrix
%   PredError] - Prediction error
%   Phi - Data vector
%   y - output from the system
%   npar - Number of estimated parameters
%   ForgFactor -Forgetting factor

	Phi=Phi(:);
	Theta=Theta(:);

% 	Estimation error

	PredError = y - Phi' * Theta;

%	Calculo de alfa

	alfa = 1;
	alfaden = Phi' * PM * Phi;
    
	if alfaden > 0.001;
		alfa = ForgFactor - (1 - ForgFactor) / alfaden;
    end

%	Inova?oes do vector de ganho (KG)

	KG = ( PM * Phi ) / ( 1 + Phi' * PM * Phi * alfa );

%	Inova?oes da matriz das covariancias

	PM = ( eye(npar, npar) - KG * Phi' * alfa ) * PM;

%	Inova?oes do Vector dos Parametros Estimados

	Theta = Theta + KG * PredError;


%   EOF: RLSDF.M




