% define_reactor.m
%
% Yeast Fermentation Reactor
%
% Romulo Antao - 2015
%
% Last Revison -  14/8/2015

% Parameters taken from 
% Computationally Efficient Model Predictive Control Algorithms
% Page 69

% Model Parameters
Fi    = 51;
Fe    = 51;
T_in  = 25;
cS_in = 60;

% Simulation parameters

h = 1;                  % Sampling Time - timescale defined in hours
 
step_length = 400;       % 400h between setpoint changes

UHigh =  200.0;          % Control signal maximum value (200 Flow Rate)
ULow  =   0.0;           % Control signal minimum value (  0 Flow Rate)

RHigh =  36.0;           % Reference signal maximum value (36 ???C)
RLow  =  22.0;           % Reference signal minimum value (22 ???C)

DHigh =   0.0;           % Disturbance signal at input maximum value ()
DLow  =   0.0;           % Disturbance signal at input minimum value ()

% Simulink Model

sim_model = 'reactor';         % Name of SIMULINK model
integrator= 'ode45';		% Name of dif. eq. solver (f. ex. ode45 or ode15s)
simoptions = simset('Solver',integrator,'MaxDataPoints',0); % Set integrator
eval(['[sizes,x0] = ' sim_model '([],[],[],0);']);    % Get initial states
set_param(sim_model, 'InitInArrayFormatMsg', 'None')

% EOF : define_reactor.m