% GPC using ARX model
%
% Last Revision
% 2 January 2015
%
% licensed under the Creative Commons Attribution-NonCommercial 4.0 
%
% Romulo Antao
% romuloantao@ua.pt

close all
clear all
clc 

%% Add required folders to the path

% Add dependencies to path 
toolbox = './deps';
model   = './model';

path(path,toolbox);
path(path,model);

%% Setup the simulink model
define_coupled_tank

%% Training related variables
load ./data/MODEL_SIGNALS_2s

%% System Parameters

na = 2;
nb = 2;

%% Online training parameters

%RLS
npar = 4;               % Number of estimated parameters
ForgFactor = 0.995;      % Forgetting Factor
P0 = 100;               % PMatrix initial value
P = P0 * eye(npar);		% PMatrix initialization
th = 0.1*rand(1,npar);  % Estimated parameters initial values  
Phi = zeros(1, npar);   % Regression vector initial values
    
%% Actuate over the system

tic

t = (0:N)*h;
simEndPoint = N-1;

fprintf(1,'Training the model:      ');

for k = 5 : simEndPoint
    
    % Progress status
    fprintf(1,'\b\b\b\b\b\b  %.2d %%',floor(k/simEndPoint*100));
       
    % Train the model
    [th, P, error(k)] = rlsdf(th, Phi, P, y(k), npar, ForgFactor);
   
    % Model estimation
    y_hat(k) = th(1)*u(k-1) + th(2)*u(k-2) + th(3)*y(k-1) + th(4)*y(k-2);
            
    % Regression vector update       
    Phi(1)=u(k);
    Phi(2)=u(k-1);
    Phi(3)=y(k);
    Phi(4)=y(k-1);
    
    % For diagnosis purposes  
    for i = 1 : npar
        Theta(k, i) = th(i);
        LongP(k, i) = P(i,i);
    end
        
end

fprintf(1,'\b\b\b\b\b\b 100 %% \r');

MSE = sum(error.*error)/(length(error));

fprintf(1,'Model MSE = %.2d \n\r', MSE);

toc
save ./data/FORWARD_ARX th
%% Plots

figure(1)
subplot(2,1,1);
plot(t(1:simEndPoint), [y_hat(1:simEndPoint)' y(1:simEndPoint)]), title('Model estimation v System output'), grid, ylabel('Output'), xlabel('s)');
legend('Output-hat','Output')
subplot(2,1,2);
plot(t(1:simEndPoint), error(1:simEndPoint),'r'), title('Error'), grid, xlabel('Time (s)');
set(gcf,'Name','1 - Model vs Plant Output','NumberTitle','off');

figure(2)
subplot(2,1,1);
plot(t(1:simEndPoint), [y_hat(1:simEndPoint)' y(1:simEndPoint)]), title('Model estimation v System output'), grid, ylabel('Output'), xlabel('s)');
legend('Output-hat','Output')
subplot(2,1,2);
plot(t(1:simEndPoint), u(1:simEndPoint),'r'), title('Control Signal'), grid, xlabel('Time (s)');
set(gcf,'Name','2 - Plant Output vs Control Signal','NumberTitle','off');

figure(3)
plot(LongP(1:simEndPoint,:)),title('Covariance matrix'), grid,
set(gcf,'Name','3 - System ID - Covariance matrix','NumberTitle','off');

figure(4)
plot(Theta(1:simEndPoint,:)),title('Model Parameters'), grid,
set(gcf,'Name','4 - System ID - Model Parameters','NumberTitle','off');
legend('num(1)','num(2)','den(1)','den(2)')

