% licensed under the Creative Commons Attribution-NonCommercial 4.0 

function [u,u_r,u_l] = gpc_type2(u,y,r,du,na,nb,C_r,C_l,wr,wl,Np,Nu,lambda)

    % Set input data columnwise
    u=u(:); y=y(:); r=r(:);

    %determinar os parametros al, ar, bl, br
    for i=1:nb
        br(:,i) = C_r(:,i);
        bl(:,i) = C_l(:,i);
    end

    for i=1:na
        ar(:,i) = C_r(:,nb+i);
        al(:,i) = C_l(:,nb+i);
    end

    % Multiplicar pelos graus de activacao wr e wl
    br_w = wr*br/sum(wr);
    ar_w = [1 -wr*ar/sum(wr)];

    bl_w = wl*bl/sum(wl);
    al_w = [1 -wl*al/sum(wl)];
    
    na = na + 1;
    nb = nb;

    %% GPC for u_r
    a_tilde = [ar_w 0] - [0 ar_w];

    % Implementacao Eq. 6.20 Jerome
    f = zeros(Np,na);
    f(1,:) = -a_tilde(2:end);

    for p=2:Np
        for i=1:na-1
            f(p,i)=f(p-1,i+1) - f(p-1,1)*a_tilde(i+1);
        end
        f(p,na)= - f(p-1,1)*a_tilde(na+1);
    end

    % Implementacao Eq. 6.21
    g = zeros(Np,Np+nb-1);
    g(1,:) = [br_w zeros(1,Np-1)];
   
    for p=2:Np
        g(p,:) = g(p-1,:);
        for i=1:nb
            g(p,p+i-1) = g(p-1,p+i-1) +  f(p-1,1)*br_w(i);
        end
    end

    % Implementacao de G segundo pagina 132 Jerome
    G = zeros(Np,Nu);

    for pp=1:Np
        i=pp;
        j=1;
        while(i>=1&&j<=Nu)
            G(pp,j) = g(pp,i);
            i=i-1;
            j=j+1;
        end
    end
    
    % Implementacao de F segundo pagina 132 jerome
    F = f;

    % Implementacao de L segundo pagina 132 jerome
    L = zeros(Np,1);

    for p=1:Np
        L(p,1) = g(p,p+1);
    end

    % Get the control signal

    K = eye(Nu)/(G'*G+lambda*eye(Nu))*G';

    r_params  =  K(1,:);
    y_params  = -K(1,:)*F;
    u_params  = -K(1,:)*L;

    u_r = r_params*r(:)+y_params*y(:)+u_params*du;


    %% GPC for u_l
    a_tilde = [al_w 0] - [0 al_w];

    %Implementacao Eq. 6.20 Jerome
    f = zeros(Np,na);
    f(1,:) = -a_tilde(2:end);

    for p=2:Np
        for i=1:na-1
            f(p,i) = f(p-1,i+1) - f(p-1,1)*a_tilde(i+1);
        end
        f(p,na) = - f(p-1,1)*a_tilde(na+1);
    end

    % Implementacao Eq. 6.21
    g = zeros(Np,Np+nb-1);
    g(1,:) = [bl_w zeros(1,Np-1)];
   
    for p=2:Np
        g(p,:) = g(p-1,:);
        for i=1:nb
            g(p,p+i-1) = g(p-1,p+i-1) +  f(p-1,1)*bl_w(i);
        end
    end

    % Implementacao de G segundo pagina 132 Jerome
    G = zeros(Np,Nu);

    for pp=1:Np
        i=pp;
        j=1;
        while(i>=1&&j<=Nu)
            G(pp,j) = g(pp,i);
            i=i-1;
            j=j+1;
        end
    end
    
    % Implementacao de F segundo pagina 132 jerome
    F = f;

    % Implementacao de L segundo pagina 132 jerome
    L = zeros(Np,1);

    for p=1:Np
        L(p,1) = g(p,p+1);
    end

    % Get the control signal

    K = eye(Nu)/(G'*G+lambda*eye(Nu))*G';

    r_params  =  K(1,:);
    y_params  = -K(1,:)*F;
    u_params  = -K(1,:)*L;

    u_l = r_params*r(:)+y_params*y(:)+u_params*du;

    % Compute the effective control signal
     u = u(1) + (u_r+u_l)/2;

end