%% psi_tsk_type2.m

%% =======  Compute the PSI matrix ========
%% (The derivative of each network output (y) with respect to each weight)

%% create the psi vector for the recursive levenberg maquardt algorithm

%% M1,M2, sigma are Rxn matrix denotes the mean and std of
%% antecedent Gaussian MFs (R rules, with ant antecedent in each rule)
%% C, S are Rx(ant+1) matrix denoting the center and spread of consequents para.
%% X is input matrix, L(x)ant matrix, each row is onw input.
%% D is Lx1 vector which denotes the desired output
%% Y is a Lx1 Vector wich denotes de obtained output
%%
%% licensed under the Creative Commons Attribution-NonCommercial 4.0 


function [PSI]=psi_matrix(X,M,sigma1,sigma2,r_out,l_out,I2l,I2r,wr,I1r,I1l,wl,y_r,y_l)

[R,ANT]=size(M);

d_M      = zeros(size(M));
d_sigma1 = zeros(size(sigma1));
d_sigma2 = zeros(size(sigma1));

%% Select MFs that contributed to the right-most system output

% Select the upper membership functions
LE=length(I2r);

% Update the upper membership functions that contributed to the right-most system output
for i=1:LE
    l=I2r(i);
    for k=1:ANT       
        d_M(l,k)=d_M(l,k)+0.5*(y_r(l)-r_out)/sum(wr)*wr(l)*(X(k)-M(l,k))/(sigma2(l,k)^2);
            
        d_sigma2(l,k)=d_sigma2(l,k)+0.5*(((X(k)-M(l,k))^2)/(sigma2(l,k)^3))...
                *(y_r(l)-r_out)*wr(l)/sum(wr);
    end
end

% Select the lower membership functions
LE=length(I2l);

% Update the lower membership functions that contributed to the right-most system output
for i=1:LE
    l=I2l(i);
    for k=1:ANT
        d_M(l,k)=d_M(l,k)+0.5*(y_r(l)-r_out)/sum(wr)*wr(l)*(X(k)-M(l,k))/(sigma1(l,k)^2);
            
        d_sigma1(l,k)=d_sigma1(l,k)+0.5*(((X(k)-M(l,k))^2)/(sigma1(l,k)^3))...
                *(y_r(l)-r_out)*wr(l)/sum(wr);
    end
end

%% Select MFs that contributed to the left-most system output

% Select the upper membership functions
LE=length(I1r);

% Update the upper membership functions that contributed to the left-most system output
for i=1:LE
    l=I1r(i);
    for k=1:ANT
        d_M(l,k)=d_M(l,k)+0.5*(y_l(l)-l_out)/sum(wl)*wl(l)*(X(k)-M(l,k))/(sigma2(l,k)^2);
            
        d_sigma2(l,k)=d_sigma2(l,k)+0.5*(((X(k)-M(l,k))^2)/(sigma2(l,k)^3))...
                *(y_l(l)-l_out)*wl(l)/sum(wl);
    end
end

% Select the lower membership functions
LE=length(I1l);

% Update the lower membership functions that contributed to the left-most system output
for i=1:LE
    l=I1l(i);
    for k=1:ANT
        d_M(l,k)=d_M(l,k)+0.5*(y_l(l)-l_out)/sum(wl)*wl(l)*(X(k)-M(l,k))/(sigma1(l,k)^2);
            
        d_sigma1(l,k)= d_sigma1(l,k)+0.5*(((X(k)-M(l,k))^2)/(sigma1(l,k)^3))...
                *(y_l(l)-l_out)*wl(l)/sum(wl);
    end
end

%% Reconstruct PSI matrix

PSI = [reshape(d_M,R*ANT,1);reshape(d_sigma1,R*ANT,1);...
       reshape(d_sigma2,R*ANT,1)];
