function [Theta, PM, PredError] = rlsdf(Theta, Phi, PM, y, npar, ForgFactor,fL)

%   RLSDF.M
%
%   Alexandre Manuel Mota - Outubro de 1997
%
%   Last Revision - 04/October/2007
%
%   [Theta, PM, PredError] = rlsdf(Theta, Phi, PM, y, npar, ForgFactor, fL)
%
%   Theta - estimated parameter
%   PM - Covariance matrix
%   PredError] - Prediction error
%   Phi - Data vector
%   y - output from the system
%   npar - Number of estimated parameters
%   ForgFactor -Forgetting factor

%   fL        - firing level (adapted to TSK local learning)
%
% licensed under the Creative Commons Attribution-NonCommercial 4.0 

	Phi=Phi(:);
	Theta=Theta(:);

% 	Estimation error

	PredError = sqrt(fL)*(y - Phi' * Theta);

%	Calculo de alfa

	alfa = 1;
    
   	alfaden = fL * Phi' * PM * Phi;
	if alfaden > 0.001;
		alfa = ForgFactor - (1 - ForgFactor) / alfaden;
	end

%	Inovacoes do vector de ganho (KG)

	KG = (fL * PM * Phi ) / ( 1 + fL * Phi' * PM * Phi * alfa );

%	Inovacoes da matriz das covariancias

	PM = ( eye(npar, npar) - KG * Phi' * alfa ) * PM;

%	Inovacoes do Vector dos Parametros Estimados

	Theta = Theta + KG * PredError;

%   EOF: RLSDF.M




