%% t2tsk_bp.m

%% apply the backpropagation training algorithm to the network parameters

%% M1,M2, sigma are Rxn matrix denotes the mean and std of
%% antecedent Gaussian MFs (R rules, with ant antecedent in each rule)
%% C, S are Rx(ant+1) matrix denoting the center and spread of consequents para.
%% X is input matrix, L(x)ant matrix, each row is onw input.
%% D is Lx1 vector which denotes the desired output
%% Y is a Lx1 Vector wich denotes de obtained output
%%
%% licensed under the Creative Commons Attribution-NonCommercial 4.0 


function [M,sigma1,sigma2]=t2tsk_bp(x,y_hat,y,M,sigma1,sigma2,r_out,l_out,I2l,I2r,wr,I1l,I1r,wl,y_r,y_l,eta)

[R,ANT]=size(M);

dE_M      = zeros(size(M));
dE_sigma1 = zeros(size(sigma1));
dE_sigma2 = zeros(size(sigma1));

error = (y_hat-y);

%% Select MFs that contributed to the right-most system output

% Select the upper membership functions
LE=length(I2r);

for i=1:LE
    l=I2r(i);
    for k=1:ANT       
        dE_M(l,k)=dE_M(l,k)+0.5*error*(y_r(l)-r_out)/sum(wr)*wr(l)*(x(k)-M(l,k))/(sigma2(l,k)^2);
            
        dE_sigma2(l,k)=dE_sigma2(l,k)+0.5*error*(((x(k)-M(l,k))^2)/(sigma2(l,k)^3))...
                *(y_r(l)-r_out)*wr(l)/sum(wr);
    end
end

% Select the lower membership functions
LE=length(I2l);

% Update the lower membership functions that contributed to the right-most system output
for i=1:LE
    l=I2l(i);
    for k=1:ANT
        dE_M(l,k)=dE_M(l,k)+0.5*error*(y_r(l)-r_out)/sum(wr)*wr(l)*(x(k)-M(l,k))/(sigma1(l,k)^2);
            
        dE_sigma1(l,k)=dE_sigma1(l,k)+0.5*error*(((x(k)-M(l,k))^2)/(sigma1(l,k)^3))...
                *(y_r(l)-r_out)*wr(l)/sum(wr);
    end
end

%% Select MFs that contributed to the left-most system output

% Select the upper membership functions
LE=length(I1r);

% Update the upper membership functions that contributed to the left-most system output
for i=1:LE
    l=I1r(i);
    for k=1:ANT
        dE_M(l,k)=dE_M(l,k)+0.5*error*(y_l(l)-l_out)/sum(wl)*wl(l)*(x(k)-M(l,k))/(sigma2(l,k)^2);
            
        dE_sigma2(l,k)=dE_sigma2(l,k)+0.5*error*(((x(k)-M(l,k))^2)/(sigma2(l,k)^3))...
                *(y_l(l)-l_out)*wl(l)/sum(wl);
    end
end

% Select the lower membership functions
LE=length(I1l);

% Update the lower membership functions that contributed to the left-most system output
for i=1:LE
    l=I1l(i);
    for k=1:ANT
        dE_M(l,k)=dE_M(l,k)+0.5*error*(y_l(l)-l_out)/sum(wl)*wl(l)*(x(k)-M(l,k))/(sigma1(l,k)^2);
            
        dE_sigma1(l,k)= dE_sigma1(l,k)+0.5*error*(((x(k)-M(l,k))^2)/(sigma1(l,k)^3))...
                *(y_l(l)-l_out)*wl(l)/sum(wl);
    end
end

M = M - eta*dE_M;
sigma1 = sigma1 - eta*dE_sigma1;
sigma2 = sigma2 - eta*dE_sigma2;

% [M1,M2,sigma] = T2FS_ant_reorder(M1,M2,sigma,R,ant);
        