%% tsk_type1.m

% Compute the output(s) of an Type-1 TSK FLS when the antecedent membership
% functions have Gaussian.

% M,sigma are mxn matrix denotes the mean and std of
% antecedent Gaussian MFs (m rules, with n antecedent in each rule)
% C are mx(n+1) matrix, which denotes the center and spread of coefficent of parameters
% X is input matrix, 1xn matrix.
%
% licensed under the Creative Commons Attribution-NonCommercial 4.0 

function [out_tsk,fL,y_i]=tsk_type1(X,M,sigma,C,REG)

    [R,ANT] = size(M);

    % Initialize the upper and lower firing levels of each rule
    fL = [];
        
    % Evaluate the membership degree of the input in each rule i
    for i = 1:R
        
        % Initialize each rule upper and lower firing level
        mU = 1;
        
        % Evaluate the membership degree of each input j their respective antecedent    
        for j = 1:length(REG)
            % Setup the gaussian membership function parameters
            P = [sigma(i,REG(j)),M(i,REG(j))];     
            % Get the membership degree 
            mu = gausstype1(X(REG(j)),P);
            % Implement the product t-norm of the antecedents firing level
            mU = min(mU,mu);
        end
        
        % Store the upper and lower firing levels of each system rule
        fL = [fL,mU];
    end

    % Get the output of each rule consequent

    y_i = zeros(size(C,1),1);

    for i=1:ANT    
        y_i = y_i + C(:,i) * X(i);
    end

    % Defuzzyfication using weighted average
 
    out_tsk = fL*y_i/sum(fL);

end