%% tsk_type2.m

% Compute the output(s) of an interval type-2 TSK FLS A2-C1 (type-2 
% antecedents and type-1 consequent) when the antecedent membership
% functions are Gaussian primary membership functions with uncertain
% means.

% M1,M2,sigma are mxn matrix denotes the mean and std of
% antecedent Gaussian MFs (m rules, with n antecedent in each rule)
% C,S are mx(n+1) matrix, which denotes the center and spread of coefficent of parameters
% X is input matrix, 1xn matrix.
% R1,R2, and R are the lower, upper and average output
% call the function gausstype2.m and interval_wtdavg.m
%
% licensed under the Creative Commons Attribution-NonCommercial 4.0 
%
function [out,r_out,l_out,UU,LL,I2l,I2u,wr,I1u,I1l,wl,yr,yl,sw_r,sw_l]=tsk_type2_uM(X,M1,M2,sigma,C,S,REG)

    [R,ANT] = size(M1);

    % Initialize the upper and lower firing levels of each rule
    UU=[];
    LL=[];

    % Evaluate the membership degree of the input in each rule i
    for i = 1:R

        % Initialize each rule upper and lower firing level
        Uu = 1;
        Ll = 1;

        % Evaluate the membership degree of each input j their respective antecedent    
        for j = 1:length(REG)
            % Setup the gaussian membership function parameters
            P = [sigma(i,REG(j)),M1(i,REG(j)),M2(i,REG(j))];
            % Get the upper (uu) and lower (ll) membership degrees
            [uu,ll] = gausstype2_uM(X(REG(j)),P);
            % Implement the product t-norm of the antecedents firing level
            Uu = min(Uu,uu);
            Ll = min(Ll,ll);
        end

        % Store the upper and lower firing levels of each system rule
        UU = [UU,Uu];
        LL = [LL,Ll];
    end

    % Get the output of each rule consequent

    yr = zeros(size(C,1),1);
    yl = zeros(size(C,1),1);

    for i = 1:ANT    
        yr = yr + C(:,i) * X(i) + S(:,i) * abs(X(i));
        yl = yl + C(:,i) * X(i) - S(:,i) * abs(X(i));
    end
    
    [out,l_out,r_out,I1l,I1u,I2l,I2u,wl,wr,sw_l,sw_r] = typereduction_KM(yl,yr,LL,UU);

end