%% typereduction_KM.m
%% Algorithm described in the P310 of Mendel's book
%% This is the four step iterative procedure to find the left point of the T2FS output
%%
%% licensed under the Creative Commons Attribution-NonCommercial 4.0 


function [out,l_out,r_out,I1l,I1u,I2l,I2u,wl,wr,L,R] = typereduction_KM(yl,yr,LL,UU)

    % Get input data columnwise
    yl = yl(:);
    yr = yr(:);
    LL = LL(:);
    UU = UU(:);
    
    % Sort the right limit of the consequent values in ascending order

    N=length(yl);
    [yl_s,idx]=sort(yl);

    % get each each rule upper and lower firing levels ordered according to c1
    L_s=LL(idx);
    U_s=UU(idx);

    % Calculate an initial estimate for yl considering L=N-1 (using all lower weights of the antecedents)
    l_out=sum(yl_s.*L_s)/sum(L_s);
    L=0;
    
    s=yl_s'*L_s;
    s1=sum(L_s);

    wl=zeros(1,N);
    I1u=[];
    I1l=[];

    % iteratively increase L as described in algorithm step 2 (P311)

    for i=1:N
        s=s-yl_s(i)*L_s(i)+yl_s(i)*U_s(i);
        s1=s1-L_s(i)+U_s(i);
        if s/s1<l_out
            l_out=s/s1;
            wl(idx(i))=U_s(i);     % register the used weight (upper in this case)
            I1u=[I1u,idx(i)];    % register the used upper firing level indexes
            L = L + 1;
        else
            wl(idx(i))=L_s(i);     % register the used weight (lower in this case)
            I1l=[I1l,idx(i)];    % register the used lower firing level indexes
        end
    end

    % Sort the right limit of the consequent values in ascending order

    N=length(yr);
    [yr_s,idx]=sort(yr);

    % get each rule upper and lower firing levels ordered according to idx
    L_s=LL(idx);
    U_s=UU(idx);

    % Calculate an initial estimate for yr considering R=0 (using all upper weights of the antecedents)
    r_out=sum(yr_s.*U_s)/sum(U_s);

    R = 0;

    s=yr_s'*U_s;
    s1=sum(U_s);

    wr=zeros(1,N);
    I2l=[];
    I2u=[];

    % iteratively increase R as described in algorithm step 2 (P311)

    for i=1:N
        s=s-yr_s(i)*U_s(i)+yr_s(i)*L_s(i);
        s1=s1-U_s(i)+L_s(i);
        if s/s1>r_out
            r_out=s/s1;
            wr(idx(i))=L_s(i);     % register the used weight (lower in this case)
            I2l=[I2l,idx(i)];    % register the used lower firing level indexes
            R = R + 1;
        else
            wr(idx(i))=U_s(i);     % register the used weight (upper in this case)
            I2u=[I2u,idx(i)];    % register the used upper firing level indexes
        end
    end

    % Finally
    out = (l_out + r_out) / 2;
    
end