function [sys, x0] = sf_coupled_tank(t,input,u,flag)

% Implementation of Couple Tank System

% S-function details here:
% http://www.mathworks.com/help/simulink/sfg/maintaining-level-1-matlab-s-functions.html#f7-13642

    %% Model Parameters

    alfa1 = 5.6186;         
    alfa2 = 5.6186;         
    alfa3 = 10;             

    A1 = 36.52;             % Cross section area (36.52 cm2)
    A2 = 36.52;              % Cross section area (36.52 cm2 )

    %% S-Function implementation
    switch flag    

        case 1, % Calculates the derivatives of the continuous state variables

            % Get system state variables
            H1 = input(1);  % Tank 1 Level
            H2 = input(2);  % Tank 2 Level

            % Get actuator states
            Q1 = u(1);      % Pump 1
            Q2 = u(2);      % Pump 2

            % Implement the dynamic model

            dH1 = 1/A1 * (Q1 - alfa1*sqrt(abs(H1)) - sign(H1-H2)*alfa3*sqrt(abs(H1-H2)) );
            dH2 = 1/A2 * (Q2 - alfa2*sqrt(abs(H2)) + sign(H1-H2)*alfa3*sqrt(abs(H1-H2)) );

            sys = [dH1; dH2];

        case 0, % Setup S function

            sys = [2 0 2 2 0 0];

%             x0 = [22.6026 17.1793];
            x0 = [32.5477 24.7382];

        case 3, % Calculates the outputs of the S-function.

            H1   = input(1);
            H2   = input(2);

            sys = [H1; H2];

        otherwise, 

            sys = [];
            
    end

end

