% Open Loop simulation of the Couple tank process
%
% 17 March 2017
%
% licensed under the Creative Commons Attribution-NonCommercial 4.0 
%
% Romulo Antao
% romuloantao@ua.pt

% NOTE:
% Simulation of second order system using a stair test signal
% To obtain a proper training sequence for TSK model extraction,
% a long test is required to ensure the training of the model at different 
% operation points.
%
close all
clear all
clc 

%% Add required folders to the path

% Set current folder
clc

% Add dependencies to path 
toolbox = '../T2FLS_toolbox';
model = './model';

path(path,toolbox);
path(path,model);

%% Setup the simulink model

define_coupled_tank

%% Control signal parameters

nelemref = 100;          % Number of steps to test
setpoint = floor(0*10 + (UHigh*10 - ULow*10)*rand(nelemref,1))/10;

setpoint = [50; setpoint];
[u,N] = gen_uSignal(setpoint,600,h);

% Generate the Disturbances
    
% SNR -48db (2bit numa ADC de 10bit)
SNR = -35;
n_steps = 10;
[In_HF_noise, Out_HF_noise, In_LF_disturbance] = gen_noise(SNR,UHigh,DLow,DHigh,n_steps,N);

In_HF_noise         = 0*In_HF_noise;
Out_HF_noise        = 0*Out_HF_noise;
In_LF_disturbance   = 0*In_LF_disturbance;
Out_LF_disturbance  = 0*In_LF_disturbance; 

%%  Variable initializations

y = 17.17*ones(N,1);
t = zeros(N,1);

%%	Open loop actuation

simEndPoint = N;

fprintf(1,'Open Loop actuation over the system...      ');
tic
for k = 3 : simEndPoint
    
    %progress status
    fprintf(1,'\b\b\b\b\b\b  %.2d %%',floor(k/simEndPoint*100));
    
	t(k) = t(k-1) + h;					% Continuous time update
    
%	Plant Output Simulation
%	ut = [T, U, LOAD];
%	READ OUTPUT FROM THE PHYSICAL SYSTEM

    ut = [t(k-1), u(k-1),Out_HF_noise(k-1), In_HF_noise(k-1), Out_LF_disturbance(k-1), In_LF_disturbance(k-1); ...
          t(k)  , u(k-1),Out_HF_noise(k-1), In_HF_noise(k-1), Out_LF_disturbance(k-1), In_LF_disturbance(k-1)];
	simoptions.InitialState=x0;
	[time,x0,output] = sim(sim_model,[t(k-1) t(k)],simoptions,ut);
	x0 = x0(length(x0),:)';
    
	y(k) = output(length(output),2)';
    H1(k) = output(length(output),1)';
    
end

fprintf(1,'\b\b\b\b\b\b100 %% \r\n');
toc
%% Plot the results

figure(1)

x_axis_handler_1(1) = subplot(211);
plot(t(1:simEndPoint),y(1:simEndPoint)), title('Output Signal'), xlabel('Time (sec)'), ylabel('Amplitude')
axis([0 t(end) 1.1*min(y) 1.1*max(y)])
x_axis_handler_1(2) = subplot(212);
plot(t(1:simEndPoint),u(1:simEndPoint)),title('Control Signal') ,xlabel('Time (sec)'), ylabel('Amplitude')
axis([0 t(end) 1.1*min(u) 1.1*max(u)])

linkaxes(x_axis_handler_1, 'x');      % Link all axes in x

%% Save some important data !!!

save ./data/MODEL_SIGNALS_2s N h y u

% EOF : open_model.m
